<?php
// $Id: drupalvb.admin-pages.inc,v 1.1.2.5 2008/05/01 20:47:36 sun Exp $

/**
 * @file
 * Drupal vB system settings.
 *
 * Most parts forked from Migrator module, http://drupal.org/project/migrator
 */

/**
 * Form builder function for DrupalvB integration settings.
 *
 * @todo Add vBulletin license.
 * @todo Add required vBulletin configuration values; remove drupalvb_get_config().
 */
function drupalvb_settings_integration() {
  if (!drupalvb_db_is_valid()) {
    drupal_goto('admin/settings/drupalvb/database');
  }

  $form = array();
  $form['drupalvb_dual_login'] = array(
    '#type' => 'checkbox',
    '#title' => t('Users login to vBulletin forum when they log into Drupal.'),
    '#default_value' => variable_get('drupalvb_dual_login', TRUE),
    '#description' => t('Select to enable the dual login feature.'),
  );

  $form['drupalvb_acct_generation'] = array(
    '#type' => 'checkbox',
    '#title' => t('Create matching vBulletin accounts for new Drupal users.'),
    '#default_value' => variable_get('drupalvb_acct_generation', TRUE),
    '#description' => t('Select to enable the matching account generation feature.'),
  );

  $form['drupalvb_acct_sync'] = array(
    '#type' => 'checkbox',
    '#title' => t('Synchronize Drupal account updates and deletions with vBulletin.'),
    '#default_value' => variable_get('drupalvb_acct_sync', TRUE),
    '#description' => t('Select to enable the account synchronization feature.'),
  );

  $zones = _system_zonelist();
  $form['drupalvb_default_timezone'] = array(
    '#type' => 'select',
    '#title' => t('Time zone'),
    '#default_value' => variable_get('drupalvb_default_timezone', 0),
    '#options' => $zones,
    '#description' => t('Select the default timezone offset for created users.'),
  );

  $roles = drupalvb_get_roles();
  $form['drupalvb_default_usergroup'] = array(
    '#type' => 'select',
    '#title' => t('Usergroup for new users'),
    '#default_value' => variable_get('drupalvb_default_usergroup', 2),
    '#options' => $roles,
    '#description' => t('Select the default usergroup for created users.'),
  );

  $form['drupalvb_default_options'] = array(
    '#type' => 'textfield',
    '#title' => t('New User Options'),
    '#default_value' => variable_get('drupalvb_default_options', '3415'),
    '#size' => 20,
    '#maxlength' => 25,
    '#description' => t('The default options set to new users who register through Drupal.'),
  );

  return system_settings_form($form);
}

/**
 * Settings form for remote system database connection.
 */
function drupalvb_settings_database() {
  $form = array();
  $db = parse_url(variable_get('drupalvb_db', is_array($GLOBALS['db_url']) ? $GLOBALS['db_url']['default'] : $GLOBALS['db_url']));
  $form['db'] = array(
    '#type' => 'fieldset',
    '#title' => t('Database connection'),
  );
  $form['db']['scheme'] = array(
    '#type' => 'radios',
    '#title' => t('Database interface'),
    '#options' => array('mysql' => 'MySQL', 'mysqli' => 'MySQLi'),
    '#default_value' => $GLOBALS['db_type'], // was: $db['scheme'],
    '#disabled' => TRUE,
    '#description' => t('Due to the database abstraction design in Drupal 5 and 6, only the current default database interface is supported.'),
  );
  $form['db']['host'] = array(
    '#type' => 'textfield',
    '#title' => t('Host'),
    '#default_value' => !empty($db['host']) ? $db['host'] : 'localhost',
    '#required' => TRUE,
  );
  $form['db']['path'] = array(
    '#type' => 'textfield',
    '#title' => t('Database'),
    '#default_value' => substr($db['path'], 1),
    '#required' => TRUE,
  );
  $form['db']['user'] = array(
    '#type' => 'textfield',
    '#title' => t('Username'),
    '#default_value' => $db['user'],
    '#required' => TRUE,
  );
  $form['db']['pass'] = array(
    '#type' => 'textfield',
    '#title' => t('Password'),
    '#default_value' => $db['pass'],
    '#required' => TRUE,
  );
  $form['db']['db_prefix'] = array(
    '#type' => 'textfield',
    '#title' => t('Table prefix'),
    '#default_value' => variable_get('drupalvb_db_prefix', 'vb_'),
  );
  $form[] = array(
    '#type' => 'submit',
    '#value' => t('Save'),
  );
  return $form;
}

function drupalvb_settings_database_submit($form_id, $form_values) {
  $url = $form_values['scheme'] .'://'. $form_values['user'] .':'. $form_values['pass'] .'@'. $form_values['host'] .'/'. $form_values['path'];
  variable_set('drupalvb_db', $url);
  variable_set('drupalvb_db_is_default', (is_array($GLOBALS['db_url']) ? $GLOBALS['db_url']['default'] == $url : $GLOBALS['db_url'] == $url));
  variable_set('drupalvb_db_prefix', $form_values['db_prefix']);
}

/**
 * Form builder function for DrupalvB actions.
 */
function drupalvb_settings_actions() {
  $form = array();
  $form['action']['action'] = array(
    '#type' => 'value',
    '#title' => 'Action',
  );
  $form['action'][] = array(
    '#type' => 'radio',
    '#name' => 'action',
    '#return_value' => 'export',
    '#title' => t('Export Drupal users to vBulletin'),
    '#description' => t('<strong>Warning:</strong> This export relies on usernames.  If an identical username exists in Drupal and vBulletin, the user account in vBulletin will be overwritten with data from Drupal (specifically: email address and password).'),
  );
  $form['action'][] = array(
    '#type' => 'radio',
    '#name' => 'action',
    '#return_value' => 'import',
    '#title' => t('Import vBulletin users into Drupal'),
    '#description' => t('Note: Importing vBulletin users is not possible, since vBulletin stores hashed passwords.  Instead, Drupal vB implements an external authentication provider in Drupal, so existing vBulletin users are registered in Drupal upon their first login in Drupal.  There is no way to automate this for all existing users.'),
    '#disabled' => TRUE,
  );
  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Start'),
  );
  return $form;
}

/**
 * Form submit callback for action form.
 */
function drupalvb_settings_actions_submit($form_id, $form_values) {
  switch ($form_values['action']) {
    case 'export':
      drupalvb_export_drupal_users();
      drupal_set_message(t('Drupal users have been exported to vBulletin.'));
      break;
  }
}

/**
 * Form builder function for DrupalvB variables (Devel).
 */
function drupalvb_settings_variables() {
  $form = array();
  $options = drupalvb_get('options');
  foreach ($options as $key => $value) {
    $form['variables'][$key]['name'] = array('#value' => check_plain($key));
    $form['variables'][$key]['value'] = array('#value' => check_plain($value));
  }
  ksort($form['variables']);
  return $form;
}

/**
 * Theme vBulletin options similar to Drupal variables (Devel).
 */
function theme_drupalvb_settings_variables(&$form) {
  $header = array(t('Name'), t('Value'));
  $rows = array();
  foreach (element_children($form['variables']) as $key) {
    $rows[] = array(
      drupal_render($form['variables'][$key]['name']),
      drupal_render($form['variables'][$key]['value']),
    );
  }
  $output = theme('table', $header, $rows);
  $output .= drupal_render($form);
  return $output;
}

